
from math import * 
import grease
import random
import pyglet

from Bait import Bait
from AI import AIEntity

class FishAI():
    def __init__(self):
        self.swimtime = 0.35 + random.random() * 0.1
        self.swimt = random.random() * self.swimtime
        self.animt = random.random()*3
        self.curdir = (0,0)

        
class Fish(AIEntity):
    COLLISION_RADIUS = 30
    COLLIDE_INTO_MASK = 0b11
    def __init__(self, world, position):
        
        self.position.position = position
        self.movement.velocity = (0,0)
        self.collision.into_mask = self.COLLIDE_INTO_MASK
        self.collision.radius = self.COLLISION_RADIUS

        names = ["redfish.png","redfish2.png"]
        a = [pyglet.resource.image(n) for n in names]       
        for n in names:
            img = pyglet.resource.image(n)
            img.anchor_x = img.width/2
            img.anchor_y = img.height/2
        a = pyglet.resource.animation("fiskanimation1.gif", flip_x=True)
        
        
        self.sprite.type = "Fish"
        self.sprite.anim_left = a 
        self.sprite.anim_right = self.sprite.anim_left.get_transform(flip_x = True)
        self.sprite.sprite = pyglet.sprite.Sprite(self.sprite.anim_left)
        self.sprite.sprite.anchor_x = 16
        self.sprite.sprite.anchor_y = 16
        
        
        #self.vis.vis = DebugLine(world,  self.position.position, self.curr_node.node.position)
        
        self.ai.active = True
        self.ai.extra = FishAI()
        
        verts = [(-20,-20),(40,0), (0,20) ]
        self.shape.verts = verts

    def on_collide(self, other, point, normal):
        if isinstance(other, Bait):
            other.delete()
        """
        color = "green"
        if self.sprite.type != "GreenFish":
            color = "red"
        cf = ColorFish(self.world, self.position.position, color)
        cf.movement.velocity.x = 100
        #print "fish collide!", other
        """
    def delete(self):
        if self.sprite.sprite._vertex_list is not None:
            self.sprite.sprite.delete() 
        AIEntity.delete(self)
    
    def captured(self):
        self.collision.into_mask = 0
        self.collision.from_mask = 0
        self.ai.active = False;
        self.movement.velocity = (0,0)
        self.movement.accel = (0,0)
    
        
    def doAI(self, dt):
            
        if not self.ai.active: 
            return
            
        self.ai.extra.animt +=dt
        self.position.angle = 5 - sin(self.ai.extra.animt*2) * 10
        self.ai.extra.swimt += dt
        if self.ai.extra.swimt > self.ai.extra.swimtime:
            self.ai.extra.swimt = -self.ai.extra.swimtime
            self.ai.extra.curdir = None
            
        if self.ai.extra.swimt < 0:
            self.movement.accel = (0,0)
            self.movement.velocity *= 0.91
            return 
            
        if self.ai.extra.curdir == None:
            self.ai.extra.curdir = 1
            if self.position.position.get_length_sqrd() > 400**2 and random.random() < 0.4:
                self.movement.accel.x = -self.position.position.x
                self.movement.accel.y = -self.position.position.y
            elif random.random() > 0.5 and isinstance(self, ColorFish):
                closest, cdist = None, 1000000
                for entity in self.world[Bait].entities:
                    distsq = entity.position.position.get_dist_sqrd(self.position.position)
                    if distsq < cdist:
                        closest = entity
                        cdist = distsq
                        
                if closest:
                    dir = (closest.position.position - self.position.position).normalized()
                    self.movement.accel = dir*300
                    
            else:
                self.movement.accel.x = 150 - random.random() * 300
                self.movement.accel.y = 150 - random.random() * 300
                    
    def choose_next_node(self):
        nodelist = self.curr_node.node.nextnodes
        if nodelist == []:
            nodelist = self.curr_node.node.parentnodes
            
       
        distsq = (self.position.position - self.curr_node.node.position).get_length_sqrd()
        if distsq < 100**2:
            self.curr_node.node = random.choice(nodelist)
            print "asda", self.curr_node.node.position, nodelist
            #print self.curr_node.node
            
    def move_to_node(self, dt):
        """
        dir = (self.curr_node.node.position - self.position.position).normalized()
        #print dir
        self.physics.body.apply_force(dir*30)
        self.physics.body.apply_force((0, 2))
        
        if self.physics.body.velocity.get_length_sqrd() > 100**2:
            break_dir = -self.physics.body.velocity;
            self.physics.body.apply_force( break_dir*1.0 )
            #print "breakgin"
        #self.physics.body.apply_force((0, 200))
        """

class ColorFish(Fish):
    COLLISION_RADIUS = 30
    COLLIDE_INTO_MASK = 0b11
    def __init__(self, world, position, color):
        self.position.position = position
        self.movement.velocity = (0,0)
        self.collision.into_mask = self.COLLIDE_INTO_MASK
        self.collision.radius = self.COLLISION_RADIUS

        if color == "green":
            self.sprite.type = "GreenFish"
            a = pyglet.resource.animation("fiskanimation1.gif", flip_x=True)
        else:
            self.sprite.type = "RedFish"
            a = pyglet.resource.animation("fiskanimation2.gif", flip_x=True)
        
        
        self.sprite.anim_left = a
        self.sprite.anim_right = self.sprite.anim_left.get_transform(flip_x = True)
        self.sprite.sprite = pyglet.sprite.Sprite(self.sprite.anim_left)
        
        for img in self.sprite.anim_left.frames:
            img.image.anchor_x = 40
        for img in self.sprite.anim_right.frames:
            img.image.anchor_x = 30
        
        self.ai.active = True
        self.ai.extra = FishAI()
        
        verts = [(-20,20),(20,20), (0,40) ]
        self.shape.verts = verts
        #self.renderable.color = "#ffa"

    
class Shark(Fish):
    def __init__(self, world, position):
        self.position.position = position
        self.movement.velocity = (0,0)
        self.collision.into_mask = self.COLLIDE_INTO_MASK
        self.collision.radius = self.COLLISION_RADIUS

        a = pyglet.resource.animation("hajanimation.gif", flip_x=True)
        
        self.sprite.type = "Shark"
        self.sprite.anim_left = a
        self.sprite.anim_right = self.sprite.anim_left.get_transform(flip_x = True)
        self.sprite.sprite = pyglet.sprite.Sprite(self.sprite.anim_left)
  
        for img in self.sprite.anim_left.frames:
            img.image.anchor_x = 40
        for img in self.sprite.anim_right.frames:
            img.image.anchor_x = 30
        
        self.ai.active = True
        self.ai.extra = FishAI()
        
        verts = [(-30,50),(-20,81), (80,50) ]
        self.shape.verts = verts
        #self.renderable.color = "#fff"
        
        
    def on_collide(self, other, point, normal):
        #Fish.on_collide(self, other, point, normal)
        if self.ai.active and isinstance(other, ColorFish):
            other.delete()
        print "fish collide!", other
