import grease
import pyglet
import random

from grease.controls import KeyControls
from grease import renderer, component, controller, collision
from pyglet.window import key

#from TitleScreen import TitleScreen
from Const import Globals
from Renderer import SpriteRenderer, FpsRenderer, SpriteComponent, Hud, Background
from Bait import *
from Fish import *
from AI import *
from Net import *

class GameScreen(grease.World):

    def configure(self):

        self.components.position = component.Position()
        self.components.movement = component.Movement()
        self.components.collision = component.Collision()     
        self.components.shape = component.Shape()
        self.components.renderable = component.Renderable()
        
        self.components.sprite = SpriteComponent()
            
            
        self.components.ai = component.Component(
            active=bool,
            extra=object
            )
        self.components.fishing = component.Component(captured_fishes=object)
        
        
        self.renderers.background = Background()
        self.renderers.camera = renderer.Camera(position=(Globals.window.width / 2, Globals.window.height / 2))
        
        self.renderers.vector = renderer.Vector(line_width=1.5)
        self.renderers.sprite = SpriteRenderer()
        
        #self.renderers.fps = FpsRenderer()
        self.renderers.hud = Hud()
        
        self.systems.movement = controller.EulerMovement()
        self.systems.collision = collision.Circular(
            handlers=[collision.dispatch_events])
        self.systems.game = GameSystem()
        self.systems.ai = AISystem()
        self.systems.fishing = FishingSystem()

        #Fish2(self, (10,10))

        Bait(self, (10,10))
        Shark(self, (-310,110))

        
        #Fish(self, (-30,50), self.systems.ai.nodes[0])
        #Crane(self, (-50, -20), 100.0)
        for i in range(10):
            x = random.choice([-1,0, 1]) * Globals.window.width / 4 #random.randint(Globals.window.width / 2, Globals.window.width / 2)
            y = random.randint(- Globals.window.height / 3, Globals.window.height / 3)
            ColorFish(self, (x, y), "green")
            
        for i in range(10):
            x = random.choice([-1,0, 1]) * Globals.window.width / 4 #random.randint(Globals.window.width / 2, Globals.window.width / 2)
            y = random.randint(- Globals.window.height / 3, Globals.window.height / 3)
            ColorFish(self, (x, y), "red")

        #Fish(self, (-30,50), self.systems.ai.nodes[1])
        #Fish(self, (-30,50), self.systems.ai.nodes[1])
        #Fish(self, (-30,50), self.systems.ai.nodes[2])
        #for x in range(500):
        #    Fish2(self, (-30,-50))


class FishingSystem(grease.System):
    
    
    def set_world(self, world):
        self.world = world
        
    def capture(self, net):
        if not net.exists:
            return
        for fish in net.fishing.captured_fishes:
            self.world.systems.game.score += 1
            fish.delete()
            #print "captured and deleted", fish
            
        net.fishing.captured_fishes = []

        # delete a shark
        for shark in self.world[Shark].entities:
            shark.movement.accel.x = 1000
            shark.ai.active = False
            break
            
        # spawn some fishing
        for i in range(4):
            y = random.randint(- Globals.window.height / 3, Globals.window.height / 3)
            cf = ColorFish(self.world, (-500,y), "red")
            cf.movement.velocity.x = 1000
            cf = ColorFish(self.world, (500,y), "green")
            cf.movement.velocity.x = -1000
    
        self.world.systems.game.new_round()
    
    def can_win(self):
        green_to_catch = self.world.systems.game.green_to_catch
        red_to_catch = self.world.systems.game.red_to_catch
        
        for e in self.world[ColorFish].entities:
            if e.sprite.type == "GreenFish":
                green_to_catch -= 1
            elif e.sprite.type == "RedFish":
                red_to_catch -= 1
        return green_to_catch <= 0 and red_to_catch <= 0
    
    def step(self, dt):
    
        if not self.can_win():
            self.world.systems.game.game_over = True
            self.world.renderers.hud.set_gameover()
    
    
        for entity in tuple(self.world[Shark].entities):
            
            if abs(entity.position.position.x) > 1000:
                entity.delete()
                print "out of bounds", entity
        
class GameSystem(KeyControls):

    def set_world(self, world):
        self.world = world
        self.score = 0
        self.baits_left = 100
        self.time_to_spawn = 10
        self.shark_chance = 2
        self.world.clock.schedule_once(self.spawn_fish, self.time_to_spawn)
        self.round = 0
        self.new_round()
        self.game_over = False
        
    
    def new_round(self):
        self.fishing = False
        self.round += 1
        greens = random.randint(self.round, self.round+3)
        reds = random.randint(self.round, self.round+3)
        self.green_to_catch = greens
        self.red_to_catch = reds
        self.world.renderers.hud.display_mission()
        self.world.clock.schedule_once(self.enable_fishing, 4)
        
    def enable_fishing(self, dt):
        self.fishing = True
        
    def spawn_fish(self, dt):
        self.world.clock.schedule_once(self.spawn_fish, self.time_to_spawn)
        
        x = random.choice([-1, 1]) * Globals.window.width /2 
        y = random.randint(- Globals.window.height / 2, Globals.window.height / 2 + 200)
        
        if random.random() < self.shark_chance:
            #print "spawn shark!"
            Shark(self.world, (x,y))
        else:
            #print "spawn fish"
            cf = ColorFish(self.world, (-500,-200), "red")
            cf.movement.velocity.x = 1000
            
        
    @KeyControls.key_press(key.ESCAPE)
    def start_single_player(self):
        Globals.window.pop_mode()
        

    def on_mouse_release(self, x, y, button, modifiers):
        #print x,y
        p = self.screen2world(x,y)
        #print p
        if not self.fishing or self.game_over:
            return
        if button & pyglet.window.mouse.LEFT:
            if len(self.world[Net].entities) == 0:
                Net(self.world, p)
        elif button & pyglet.window.mouse.RIGHT:
            if (self.baits_left > 0):
                Bait(self.world, p)
                self.baits_left -= 1
        
        
        self.score += 1

    def screen2world(self, x,y):
        campos = self.world.renderers.camera.position
        return x-campos[0], y-campos[1]

    


