import grease
import pyglet

from Const import Globals

class FpsRenderer(grease.Renderer):
    def __init__(self):
        self.fps_display = pyglet.clock.ClockDisplay()
        
    def draw(self):
        pyglet.gl.glLoadIdentity()
        self.fps_display.draw()

class SpriteComponent(grease.component.Component):
    def __init__(self):
        grease.component.Component.__init__(self, 
            sprite=object,
            anim_left=object,
            anim_right=object,
            type=str,
            )
        
class SpriteRenderer(grease.Renderer):
    def __init__(self):
        #self.images = {}
        #self.sprites = {}
        self.batches = {}
        
    def set_world(self, world):
        self.world = world
        
    def draw(self):
        #todo: optimize me? use sprite + batch?
        #todo: add support for rotations and more
        for p, s,m in self.world.components.join("position", "sprite", "movement"):
            if not s.type in self.batches:
                self.batches[s.type] = pyglet.graphics.Batch()
            if s.sprite.batch == None:
                s.sprite.batch = self.batches[s.type]
                
            s.sprite.position = p.position.x, p.position.y
            s.sprite.rotation = p.angle
            if s.anim_left and s.anim_right:
                if m.velocity.x > 0 and s.sprite.image != s.anim_left:
                    s.sprite.image = s.anim_left
                elif m.velocity.x < 0 and s.sprite.image != s.anim_right:
                    s.sprite.image = s.anim_right
        
        for b in self.batches.values():
            b.draw()
            
class Background(grease.Renderer):
    def set_world(self, world):
        self.world = world
        self.bg_img = pyglet.resource.image("background.png")
        
    def draw(self):
        self.bg_img.blit(0,0)


    
class Hud(grease.Renderer):
    def set_world(self, world):
        self.world = world
        self.last_score = None
        self.last_baits_left = None
        self.gameover_labels = []
        
    def display_mission(self):
        green = self.world.systems.game.green_to_catch
        red = self.world.systems.game.red_to_catch
        round = self.world.systems.game.round
        mission_texts = ["Round %i" % round, "Capture %i green and %i red fishes" % (green, red)]
        self.mission_labels = []
        y =0
        for t in mission_texts:
            
            self.mission_labels.append(
                pyglet.text.Label(
                    t,
                    color=(255, 255, 255, 255),
                    font_size=35, bold=True,
                    x=0, y=y, 
                    anchor_x='center', anchor_y='center')
                )
            y -=50
        self.world.clock.schedule_once(self.fade_mission, 1)
        print Globals.window.width
    
    
    def fade_mission(self, dt):
        for l in tuple(self.mission_labels):
            c = l.color
            l.color = (c[0],c[1],c[2],c[3]-1)
            
            if l.color[3] < 10:
                self.mission_labels.remove(l)
            else:
                self.world.clock.schedule_once(self.fade_mission, 0.5)
        
        
    def set_gameover(self):
        self.gameover_labels = [pyglet.text.Label(
                    "Game Over!",
                    color=(255, 255, 255, 255),
                    font_size=35, bold=True,
                    x=Globals.window.width/2, y=50, 
                    anchor_x='center', anchor_y='center')
                ,
                pyglet.text.Label(
                    "You made it to round %i!" % self.world.systems.game.round,
                    color=(255, 255, 255, 255),
                    font_size=16, bold=True,
                    x=Globals.window.width/2, y=20, 
                    anchor_x='center', anchor_y='center')
                ,
                pyglet.text.Label(
                    "(Press Esc to return to menu)",
                    color=(255, 255, 255, 255),
                    font_size=16, bold=True,
                    x=Globals.window.width/2, y=-100, 
                    anchor_x='center', anchor_y='center')
                ]
    def draw(self):
        game = self.world.systems.game
        if game.game_over:
            if self.gameover_labels == []:
                self.set_gameover()
            for l in self.gameover_labels:
                l.draw()
        else:
            for l in self.mission_labels:
                l.draw()
        
        
        
        